@@ -1,13 +1,12 @@
-package main;
-BEGIN {
-  $main::VERSION = '0.11';
+package App::redisp;
+{
+  $App::redisp::VERSION = '0.13';
 }
+package # hide
+        main;
 sub eval_ctx { eval "sub { $_[0] }" } # Here to avoid any closures
 
 package App::redisp;
-BEGIN {
-  $App::redisp::VERSION = '0.11';
-}
 # ABSTRACT: Perl redis shell
 
 use B qw(svref_2object);
@@ -21,11 +20,11 @@ use Tie::Redis;
 use constant HAVE_READKEY => eval { require Term::ReadKey };
 
 use App::redisp::Commands qw(@COMMANDS);
-use App::redisp::EvalWithLexicals; # Just a hacked copy of Eval::WithLexicals
+use Eval::WithLexicals;
 
 has eval_with_lexicals => (
   is => 'ro',
-  default => sub { App::redisp::EvalWithLexicals->new(
+  default => sub { Eval::WithLexicals->new(
       in_package => 'main'
     );
   }
@@ -66,7 +65,7 @@ my %special = (
   keyword => sub {
     my($param) = @_;
     if($param =~ /^\s*(\w+)\s+([^\%\@\$].*)/) {
-      $param = "redis_raw(q{$1}, $2)";
+      $param = "redis(q{$1}, $2)";
       debug "Replaced with '$param'\n";
     }
     return $param;
@@ -195,13 +194,25 @@ sub _install_commands {
   for my $cmd(@COMMANDS) {
     next if exists $redis_special_commands{$cmd};
     *{"main::$cmd"} = sub(@) {
-      $self->redis->$cmd(@_)->recv;
+      my @items = $self->redis->{_conn}->$cmd(@_);
+
+      if(@items == 1 && ref $items[0] eq 'ARRAY') {
+        return @{$items[0]};
+      } else {
+        return @items;
+      }
     };
   }
 
-  *{"main::redis_raw"} = sub(@) {
+  *{"main::redis"} = sub(@) {
     my($cmd, @args) = @_;
-    $self->redis->$cmd(@args)->recv;
+    my @items = $self->redis->{_conn}->$cmd(@args);
+
+    if(@items == 1 && ref $items[0] eq 'ARRAY') {
+      return @{$items[0]};
+    } else {
+      return @items;
+    }
   };
 }
 
@@ -244,24 +255,25 @@ sub _find_referenced {
 
 =head1 NAME
 
-main - Perl redis shell
+App::redisp - Perl redis shell
 
 =head1 VERSION
 
-version 0.11
+version 0.13
 
 =head1 SYNOPSIS
 
  $ redisp
  localhost> keys "foo*"
  "foobar", "food"
- localhost> set foobarbaz 12
+ localhost> set foobarbaz, 12
  "OK"
 
  # Or in perl style
  localhost> $foobar
  10
 
+ # Actually these next ones aren't implemented yet...
  localhost> .encoding utf-8
  localhost> .server xxx
  localhost> .reconnect
@@ -321,6 +333,39 @@ SCALAR, etc values. This application makes Perl match the Redis behaviour, it's
 invalid to use more than one type at a particular name. The error will be:
 C<ERR Operation against a key holding the wrong kind of value>.
 
+Due to the way this works it's impossible to use symbolic references (e.g.
+C<${"foo$a"}>), your code needs to reference top level keys it uses at compile
+time.
+
+=head1 EXAMPLES
+
+Yet more examples, because the synopsis section was getting sort of big.
+
+C<info> is a command that returns a hash, so to grab something like the version
+you can do this:
+
+ localhost> info
+ [returns big hash]
+
+ localhost> info->{redis_version}
+ "2.1.10"
+
+Due to some commands clashing with Perl keywords you can't use them as
+functions. C<Keys> and C<exists> is something notable for this.
+
+  localhost> keys "foo*" # Special cased
+
+  localhost> sort keys "foo*" # doesn't work as you'd expect
+
+  localhost> sort redis qw(keys foo*) # does what you wanted
+
+Pub/sub can be used, but you need to write some code yourself:
+XXX: This doesn't work at all yet!
+
+ localhost> subscribe foo, sub { print "@_\n" }
+ [prints messages, ^C stops, but you'll need to unsubscribe manually]
+ localhost> unsubscribe foo
+
 =head1 BUGS
 
 This goes I<quite> close to the internals of Perl so there may be issues with
@@ -345,8 +390,9 @@ David Leadbeater <dgl@dgl.cx>
 
 This software is copyright (c) 2011 by David Leadbeater.
 
-This is free software; you can redistribute it and/or modify it under
-the terms of the Beerware license.
+This program is free software. It comes without any warranty, to the extent
+permitted by applicable law. You can redistribute it and/or modify it under the
+terms of the Beer-ware license revision 42.
 
 =cut
 
